<script setup>
import AppLayoutCity from '@/Layouts/AppLayoutCity.vue';
import Welcome from '@/Jetstream/Welcome.vue';
import Datepicker from '@vuepic/vue-datepicker';
import JetDropdown from '@/Jetstream/Dropdown.vue';
import JetDropdownLink from '@/Jetstream/DropdownLink.vue';
import '@vuepic/vue-datepicker/dist/main.css';
import { startOfDay, endOfDay, subDays, endOfMonth, endOfYear, startOfMonth, startOfYear, subMonths } from 'date-fns';
import Highcharts from 'highcharts';

import VueMultiselect from 'vue-multiselect';
import mapboxgl from 'mapbox-gl';
import 'mapbox-gl/dist/mapbox-gl.css';
import DK_Input from '@/components/dk_Input.vue';
import DK_Button from '@/components/dk_Button.vue';
import JetModal from '@/Jetstream/Modal.vue';
import { Link } from '@inertiajs/inertia-vue3';
import { axiosApiCalls } from '../../../public/libs/datakorumJS';

</script>
<style>
table.dataTable thead th,
table.dataTable thead td {
    padding: 5px 10px;
    font-size: 0.9rem !important;
    border-bottom: 1px solid rgba(200, 200, 200, 0.75) !important;
}

.dataTables_wrapper .dataTables_paginate .paginate_button.disabled,
.dataTables_wrapper .dataTables_paginate .paginate_button.disabled:hover,
.dataTables_wrapper .dataTables_paginate .paginate_button.disabled:active {
    cursor: default;
    color: rgba(200, 200, 200, 0.9) !important;
    border: 1px solid transparent;
    background: transparent;
    box-shadow: none;
}

.dataTables_wrapper .dataTables_paginate .paginate_button {
    box-sizing: border-box;
    display: inline-block;
    min-width: 1.5em;
    padding: 0.5em 1em;
    margin-left: 2px;
    text-align: center;
    text-decoration: none !important;
    cursor: pointer;
    cursor: hand;
    color: rgba(200, 200, 200, 0.9) !important;
    border: 0;
    border-radius: 0.375rem;
}

.dataTables_wrapper .dataTables_paginate .paginate_button:hover {
    color: rgba(200, 200, 200, 1) !important;
    background-color: rgba(0, 152, 219, 0.1) !important;
    background-image: unset !important;
    border: 0;
}

.dp__action_row {
    border-radius: 0.5rem;
}

.dp__input_reg {
    background: none;
    border-width: 1px;
    border-color: rgba(200, 200, 200, 0.9);
    border-radius: 0.5rem;

}

.dp__pointer {
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
}



.dp__theme_light {
    --dp-background-color: rgba(0, 57, 93, 0.95);
    --dp-text-color: #e3e3e3;
    --dp-hover-color: rgba(0, 255, 255, 0.2);
    --dp-hover-text-color: #FFFAFA;
    --dp-hover-icon-color: #e3e3e3;
    --dp-primary-color: rgba(0, 255, 255, 0.5);
    --dp-primary-text-color: #FFFAFA;
    --dp-secondary-color: #959595;
    --dp-border-color: #FFFAFA;
    --dp-menu-border-color: #FFFAFA;
    --dp-border-color-hover: #FFFAFA;
    --dp-disabled-color: #959595;
    --dp-scroll-bar-background: #f3f3f3;
    --dp-scroll-bar-color: #959595;
    --dp-success-color: rgba(5, 255, 243, 0.7);
    --dp-success-color-disabled: rgba(5, 255, 243, 0.7);
    --dp-icon-color: #e3e3e3;
    --dp-danger-color: #e6002e;
    --dp-highlight-color: rgba(25, 118, 210, 0.1);
}

</style>

<template>
    <AppLayoutCity title="Readings">
        <div v-if="this.screen_width >= 768" style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background -z-1 overflow-y-scroll cursor-default pb-6">
            <!-- TIME INTERVAL SELECTOR -->
            <div class="mx-6 text-xs text-center text-gray-light flex justify-between">
                <div class="flex flex-col justify-start items-start w-full py-4">
                    <div class="justify-between self-center items-center flex w-full"> <!--id="routeprofile"-->
                        <p class="text-white-snow font-light text-2xl w-full text-left">
                            {{ this.data.device.name }}&nbsp;
                            (Id:&nbsp;{{ this.data.device.id }} {{ (this.data.busIdDevice) ? " - Bus Id: " + this.data.busIdDevice : ''}})
                        </p>

                        <div class="flex gap-2 items-end">
                            <div class="flex items-start justify-end" v-bind:class="{'hidden': this.data.type_name == 'PARK'}">
                                <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('profile', { id:  this.data.device.id })" >
                                    <DK_Button color="blue" :fixedSize="true" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')">
                                        {{ this.$t("general.Profile") }}
                                    </DK_Button>
                                </Link>
                            </div>
                            <div class="flex items-start justify-end">
                                <DK_Button color="blue" :fixedSize="true" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit'); window.history.go(-1)">
                                    {{ this.$t("general.Back") }}
                                </DK_Button>
                            </div>
                        </div>
                    </div>
                    <div class="mt-2 inline-flex gap-6 w-full">

                        <Datepicker id="interval_date" class="min-w-[21rem]" v-model="date" vertical
                            :presetRanges="this.presetRanges" range @update:modelValue="methodUpdateDate" :clearable="false"
                            :format="this.methodFormatDatetoDatepicker"></Datepicker>

                    </div>
                    <div class="w-full flex flex-row items-center justify-between mt-6">
                        <div class="w-fit flex" v-if="this.data.type_name != 'PARK'">
                            <DK_Button color="blue" :fixedSize="true" v-on:click="this.changeChartsMode(true);" :off="!this.charts_mode" switch="left">
                                {{ this.$t("general.Merge") }}
                            </DK_Button>

                            <DK_Button color="blue" :fixedSize="true" v-on:click="this.changeChartsMode(false);" :off="this.charts_mode" switch="right">
                                {{ this.$t("general.Disunify") }}
                            </DK_Button>
                        </div>
                        <div class="w-56">
                            <VueMultiselect v-if="this.data['meters']"
                                v-model="this.data.busIdDevice" :options="Object.keys(this.data['meters'])" :close-on-select="true"
                                :multiple="false" placeholder='Select meter' @select="selectMultiselectMeter"
                                :allow-empty="false" />
                        </div>
                    </div>
                    <div v-bind:class="{ 'hidden': this.data.type_name != null && typeof this.data.data != 'undefined' && typeof this.data.dates != 'undefined'}" class="flex gap-2 whitespace-nowrap items-baseline mt-4">
                        <i class="fa-solid fa-triangle-exclamation" style="font-size:0.8rem; color:rgba(224,191,52,1);"></i>
                        <p class="font-light text-sm text-white-snow flex whitespace-nowrap">
                            {{ this.$t('text.No records were found for the device') }}
                        </p>
                    </div>
                </div>
            </div>

            <div class="flex gap-x-6 px-6 mb-6 text-gray-light" v-if="this.data.solutions_type && this.data.solutions_type == 7">

                <div class="w-2/5 grid grid-cols-6 gap-4 2xl:gap-6">

                    <div class="baltoro-container px-6 col-span-3">

                        <div class="w-full flex justify-between items-center py-3">
                            <i class="DK-ruido" style="font-size:1.25rem; color:#FFFAFA;"></i>
                            <div class="flex gap-1 whitespace-nowrap text-lg">
                                <p>{{ this.data.data[22][0] }}</p>
                                <p>{{ this.data.units[22]}} </p>
                            </div>
                        </div>

                        <div class="w-full border-gray-semimedium border-t"></div>

                        <div class="w-full flex flex-col justify-end items-end py-2">
                            <p class="text-xs">LAeqT (1 min)</p>
                            <p class="text-xs">{{ this.limitDates(this.data.dates[0], 1) }}</p>
                        </div>

                    </div>

                    <div class="col-span-3 baltoro-container px-6">

                        <div class="w-full flex justify-between items-center py-3">
                            <i class="DK-ruido" style="font-size:1.25rem; color:#FFFAFA;"></i>
                            <div class="flex gap-1 whitespace-nowrap text-lg">
                                <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LAeq1H')}} </p>
                                <p> {{this.data.units[22]}} </p>
                            </div>
                        </div>

                        <div class="w-full border-gray-semimedium border-t"></div>

                        <div class="w-full flex flex-col justify-end items-end py-2">
                            <p class="text-xs">LAeqT (60 min)</p>
                            <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                        </div>

                    </div>

                    <div class="col-span-2 baltoro-container px-6">

                        <div class="w-full flex justify-between items-center py-3">
                            <i class="DK-ruido" style="font-size:1.25rem; color:#FFFAFA;"></i>
                            <div class="flex gap-1 whitespace-nowrap text-lg">
                                <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LA10')}} </p>
                                <p> {{this.data.units[22]}} </p>
                            </div>
                        </div>

                        <div class="w-full border-gray-semimedium border-t"></div>

                        <div class="w-full flex flex-col justify-end items-end py-2">
                            <p class="text-xs">LA10 (60 min)</p>
                            <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                        </div>

                    </div>

                    <div class="col-span-2 baltoro-container px-6">

                        <div class="w-full flex justify-between items-center py-3">
                            <i class="DK-ruido" style="font-size:1.25rem; color:#FFFAFA;"></i>
                            <div class="flex gap-1 whitespace-nowrap text-lg">
                                <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LA50')}} </p>
                                <p> {{this.data.units[22]}} </p>
                            </div>
                        </div>

                        <div class="w-full border-gray-semimedium border-t"></div>

                        <div class="w-full flex flex-col justify-end items-end py-2">
                            <p class="text-xs">LA50 (60 min)</p>
                            <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                        </div>

                    </div>

                    <div class="col-span-2 baltoro-container px-6">

                        <div class="w-full flex justify-between items-center py-3">
                            <i class="DK-ruido" style="font-size:1.25rem; color:#FFFAFA;"></i>
                            <div class="flex gap-1 whitespace-nowrap text-lg">
                                <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LA90')}} </p>
                                <p> {{this.data.units[22]}} </p>
                            </div>
                        </div>

                        <div class="w-full border-gray-semimedium border-t"></div>

                        <div class="w-full flex flex-col justify-end items-end py-2">
                            <p class="text-xs">LA90 (60 min)</p>
                            <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                        </div>

                    </div>

                </div>

                <div class="w-3/5 flex justify-center gap-2 items-center baltoro-container pl-6 pr-4 py-4">

                    <div class="grid grid-rows-5 h-full place-items-center 2xl:px-4">

                        <div></div>
                        <p class="w-full text-right text-sm">Lday</p>
                        <p class="w-full text-right text-sm">Levening</p>
                        <p class="w-full text-right text-sm">Lnight</p>
                        <p class="w-full text-right text-sm">Lden</p>

                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Mon_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(1).substring(5,10) }})</p>
                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(1), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(1), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(1), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(1), 'Lden')}} </p>
                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Tue_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(2).substring(5,10) }})</p>

                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(2), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(2), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(2), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(2), 'Lden')}} </p>
                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Wed_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(3).substring(5,10) }})</p>

                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(3), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(3), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(3), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(3), 'Lden')}} </p>
                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Thu_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(4).substring(5,10) }})</p>

                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(4), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(4), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(4), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(4), 'Lden')}} </p>
                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Fri_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(5).substring(5,10) }})</p>

                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(5), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(5), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(5), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(5), 'Lden')}} </p>
                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Sat_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(6).substring(5,10) }})</p>

                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(6), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(6), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(6), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(6), 'Lden')}} </p>
                    </div>

                    <div class="h-full pt-4 pb-2">
                        <div class="border-r border-gray-semimedium h-full"></div>
                    </div>

                    <div class="w-full grid grid-rows-5 h-full place-items-center">
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Sun_full')}}</p>
                            <p class="text-xs w-full text-center">({{ this.getDateString(0).substring(5,10) }})</p>

                        </div>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(0), 'Ld')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(0), 'Le')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(0), 'Ln')}} </p>
                        <p class="w-full text-center"> {{this.getNoiseData(this.getDateString(0), 'Lden')}} </p>
                    </div>

                </div>

            </div>

            <div class="grid 2xl:grid-cols-8 grid-cols-6 gap-x-6 px-6 mb-6" v-for="(chart, index) in this.Charts" :key="index"
                v-bind:class="{ 'hidden': chart == undefined }">


                <div class="2xl:col-span-7 col-span-5 baltoro-container" v-bind:class="{'2xl:!col-span-8 !col-span-6' : this.data.type_name == 'PARK'}">

                    <!-- CHART -->
                    <div class="px-10 pt-4">
                        <p class="text-gray-light font-light text-2xl mb-4">{{ (chart != undefined) ?
                            this.getName(chart) :
                            ''
                        }}</p>
                        <div class="2xl:col-span-7 col-span-5 cursor-crosshair pb-4"
                            v-bind:class="{ 'h-[5.5rem]': this.charts_size.includes(index) && this.data.type_name == 'PARK',' h-[11rem]': this.charts_size.includes(index) && this.data.type_name != 'PARK', 'h-[22rem]': !this.charts_size.includes(index) }">
                            <div class="h-full w-full" :id="'id_chart' + index"></div>
                        </div>
                    </div>

                </div>

                <!-- SIDEMENU -->
                <div class="col-span-1 grid content-start text-center text-gray-light text-xs gap-6"
                    v-if="this.data.type_name == 'MODBUS' || this.data.type_name == 'PULSES' || this.data.type_name == 'MBUS'">

                    <VueMultiselect v-if="this.check_VMoptions(index)" :id="'id_VueMultiselect_data_type' + index"
                        v-model="this.VMmodel[index]" :options="this.VMoptions[index]" :close-on-select="true"
                        :multiple="false" placeholder='Select sensor' @select="selectMultiselectSensor" label="name"
                        track-by="name" :allow-empty="false" />

                    <div class="m-0 p-0 baltoro-container py-1">
                        <p class="text-cyan-text font-normal py-1">{{ this.$t("water." + "maxAmpLang") }}</p>
                        <div class="w-full inline-flex justify-center items-end">
                            <p class="text-3xl font-sevensegment">
                                {{ this.max_amplitude[index] }}
                            </p>
                            &nbsp;
                            <p class="text-xl pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                        </div>
                    </div>

                    <div class="m-0 gap-2 grid text-center baltoro-container py-1">

                        <p class="col-span-2 text-cyan-text font-normal py-1">{{ this.$t("general." + "Summary statistics")
                        }}
                        </p>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Max") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-2xl font-sevensegment self-center">
                                    {{ this.max[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Avg") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-2xl font-sevensegment self-center">
                                    {{ this.avg[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Min") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-2xl font-sevensegment self-center">
                                    {{ this.min[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                    </div>
                </div>

                <div class="col-span-1 grid content-start text-center text-gray-light text-xs gap-6 hidden"
                    v-else-if="this.data.type_name == 'PARK'">

                    <div class="m-0 gap-2 grid text-center baltoro-container pt-1 pb-2">

                        <p class="col-span-2 text-cyan-text font-normal py-1">{{ this.$t("general." + "Parking time statistics") }}</p>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Max") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-sm self-center">
                                    {{ this.max[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Avg") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-sm self-center">
                                    {{ this.avg[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Min") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-sm self-center">
                                    {{ this.min[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                    </div>
                </div>
            </div>

            <div v-if="this.data.type_name == 'PARK'" class="grid 2xl:grid-cols-5 grid-cols-7 gap-x-6 px-6">

                <div class="2xl:col-span-1 col-span-2 baltoro-container flex flex-col justify-between pb-5 relative">

                    <div class="content-between m-0 cursor-default text-sm absolute top-0 left-0 right-0 z-50 px-2">
                        <div class="flex justify-between">
                            <p v-if="typeof this.data.data == 'undefined'" class="service-label bg-gray-semilight text-bluerarro mt-2 invisible">
                                {{ this.$t("general.No sendings yet") }}
                            </p>

                            <p v-else-if="this.data.data[0]['lecture_value'] == 1" class="service-label bg-Taken-spot text-bluerarro mt-2 invisible">
                                {{ this.$t("general.Taken-spot") }}
                            </p>

                            <p v-else-if="this.data.data[0]['lecture_value'] == 0" class="service-label bg-Available-spot text-bluerarro mt-2 invisible">
                                {{ this.$t("general.Available-spot") }}
                            </p>

                            <div v-if="typeof this.data.data == 'undefined'" class="flex justify-center items-center">
                                <i class="DK-signal-01" style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                <i class="DK-Batt-25 animate-pulse" style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                            </div>

                            <div v-else class="flex justify-center items-center">
                                <i v-if="this.data.data[0]['RSSI'] <= -110" class="DK-signal-01"
                                    style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else-if="this.data.data[0]['RSSI'] <= -100" class="DK-signal-02"
                                    style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else-if="this.data.data[0]['RSSI'] <= -85" class="DK-signal-03"
                                    style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else-if="this.data.data[0]['RSSI'] <= -70" class="DK-signal-04"
                                    style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else class="DK-signal-05"
                                    style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>

                                <i v-if="this.data.data[0]['Battery Level'] <= 2.9"
                                    class="DK-Batt-25 animate-pulse"
                                    style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else-if="this.data.data[0]['Battery Level'] <= 3.1"
                                    class="DK-Batt-50"
                                    style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else-if="this.data.data[0]['Battery Level'] <= 3.3"
                                    class="DK-Batt-75"
                                    style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                <i v-else class="DK-Batt-100"
                                    style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                            </div>
                        </div>
                    </div>

                    <div class="h-[15rem] w-full rounded-t-md mb-4" id="map"  v-on:click="this.openMapconfig()"/>

                        <JetModal :show="this.Mapconfig" :closeable="false">

                            <p class="text-md text-center text-cyan-text font-light p-2">
                                {{ this.$t("general." + "Edit location") }}
                            </p>

                            <div class="h-[22.5rem] w-[40rem] rounded-t-md" id="mapModal"/>

                            <div class="grid grid-cols-3 gap-4 p-2">

                                <div class="flex justify-center text-md py-1 text-white-snow font-light gap-2">
                                    <p class="text-right text-cyan-text">{{ this.$t("general." + "Latitude") }}:</p>
                                    <DK_Input id="id_map_lat" type="text" :border="false" width="6rem" :value="parseFloat(this.data.device.GPS_lat)" v-on:keyup="updateLocation()"></DK_Input>
                                </div>

                                <div class="flex justify-center text-md py-1 text-white-snow font-light gap-2">
                                    <p class="text-right text-cyan-text">{{ this.$t("general." + "Longitude") }}:</p>
                                    <DK_Input id="id_map_lng" type="text" :border="false" width="6rem" :value="parseFloat(this.data.device.GPS_lng)" v-on:keyup="updateLocation()"></DK_Input>
                                </div>

                                <div class="flex justify-end items-center">
                                    <div class="flex gap-2">

                                        <DK_Button color="blue" :fixedSize="true" v-on:click="this.SaveMap()">
                                            {{ this.$t("general.Save") }}
                                        </DK_Button>

                                        <DK_Button color="blue" :fixedSize="true" v-on:click="this.CancelMap()">
                                            {{ this.$t("general.Cancel") }}
                                        </DK_Button>

                                    </div>
                                </div>
                            </div>

                        </JetModal>

                    <div class="grid gap-0.5 px-6 mb-4">
                        <div class="flex row-span-1 justify-between text-md text-gray-light font-light gap-12">
                            <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Name") }}</p>
                            <DK_Input id="id_name" type="text" :border="false" width="full" :value="this.data.device.name" textPosition="right"></DK_Input>
                        </div>
                        <div class="flex row-span-1 justify-between text-md text-gray-light font-light gap-12">
                            <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "GatewayID") }}</p>
                            <DK_Input type="text" :border="false" width="full" :value="this.data.device.id" textPosition="right" :readonly="true"></DK_Input>
                        </div>
                        <div class="flex row-span-1 justify-between text-md text-gray-light font-light gap-12">
                            <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Model") }}</p>
                            <DK_Input type="text" :border="false" width="full" :value="this.data.device.type_name" textPosition="right" :readonly="true"></DK_Input>
                        </div>
                        <div class="flex row-span-1 justify-between text-md text-gray-light font-light gap-12">
                            <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Latitude") }}</p>
                            <DK_Input id="id_lat" type="text" :border="false" width="full" :value="parseFloat(this.data.device.GPS_lat).toFixed(6)" textPosition="right"></DK_Input>
                        </div>
                        <div class="flex row-span-1 justify-between text-md text-gray-light font-light gap-12">
                            <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Longitude") }}</p>
                            <DK_Input id="id_lng" type="text" :border="false" width="full" :value="parseFloat(this.data.device.GPS_lng).toFixed(6)" textPosition="right"></DK_Input>
                        </div>
                    </div>

                    <div class="flex justify-end items-end gap-2 px-6">

                        <DK_Button color="blue" :fixedSize="true" v-on:click="this.Save_update()">
                            {{ this.$t("general.Save") }}
                        </DK_Button>

                        <DK_Button color="blue" :fixedSize="true" v-on:click="this.Cancel_update()">
                            {{ this.$t("general.Cancel") }}
                        </DK_Button>

                    </div>

                </div>

                <div class="2xl:col-span-4 col-span-5 pt-8 px-10 baltoro-container">

                    <table id="basicTable" class="row-border">
                        <thead class="bg-transparent text-gray-light">
                            <tr>
                                <th>{{ this.$t("general." + "Date start") }}</th>
                                <th>{{ this.$t("general." + "Date end") }}</th>
                                <th>{{ this.$t("general." + "Time_mode") }}</th>
                                <th>{{ this.$t("general." + "Parkingspace") }}</th>
                            </tr>
                        </thead>
                        <tbody class="text-center">

                        </tbody>
                    </table>

                </div>

            </div>

            <div v-else-if="this.data.meters" class="grid 2xl:grid-cols-8 grid-cols-6 gap-x-6 px-6">

                <div class="2xl:col-span-8 col-span-6 pt-5 mb-6 px-10 baltoro-container min-w-fit">

                    <table id="basicTable" class="row-border">
                        <thead class="bg-transparent text-gray-light">
                            <tr>
                                <th>{{ this.$t("general." + "Date") }}</th>
                                <th v-for="(data, index) in this.data.data" :key="index">
                                    <a v-if="this.data.meters[this.data.busIdDevice].units[index] != ''">
                                        {{ this.data.meters[this.data.busIdDevice].names[index] + ' (' + this.data.meters[this.data.busIdDevice].units[index]+ ')' }}
                                    </a>
                                    <a v-else>
                                        {{ this.data.meters[this.data.busIdDevice].names[index] }}
                                    </a>
                                </th>
                            </tr>
                        </thead>
                        <tbody class="text-center">
                        </tbody>
                    </table>

                </div>

            </div>

            <div v-else class="grid 2xl:grid-cols-8 grid-cols-6 gap-x-6 px-6">

                <div class="2xl:col-span-8 col-span-6 pt-5 mb-6 px-10 baltoro-container min-w-fit">

                    <table id="basicTable" class="row-border">
                        <thead class="bg-transparent text-gray-light">
                            <tr>
                                <th>{{ this.$t("general." + "Date") }}</th>
                                <th v-for="(data, index) in this.data.data" :key="index">
                                    <a v-if="this.data.units[index] != ''">
                                        {{ this.data.names[index] + ' (' + this.data.units[index]+ ')' }}
                                    </a>
                                    <a v-else>
                                        {{ this.data.names[index] }}
                                    </a>
                                </th>
                            </tr>
                        </thead>
                        <tbody class="text-center">
                        </tbody>
                    </table>

                </div>

            </div>

        </div>
        <div v-else style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background -z-1 cursor-default select-none overflow-x-hidden">

            <div class="w-full flex flex-col justify-center items-start pt-2 pb-1 gap-2">

                <div class="w-full flex flex-row justify-between px-3">
                    <div class="w-full flex flex-col justify-start">
                        <div class="flex flex-row justify-start">
                            <p class="text-white-snow font-light text-2xl text-left">{{ this.data.device.name }}</p>
                        </div>
                        <p class="text-white-snow font-light text-sm text-left">{{ this.data.device.id }}</p>
                    </div>
                    <div class="flex justify-end gap-2">
                        <div class="flex items-center justify-center">
                            <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('meterProfile', { id:  this.data.device.id })" >
                                <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm p-1.5 text-center inline-flex items-center">
                                    <i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>
                                </button>
                            </Link>
                        </div>
                        <div class="flex items-center justify-center">
                            <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit'); window.history.go(-1)">
                                <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm p-1.5 text-center inline-flex items-center">
                                    <i class="fa-solid fa-arrow-left" style="font-size:0.75rem; color:#FFFAFA;"></i>
                                </button>
                            </Link>
                        </div>
                    </div>
                </div>

                <div class="w-full px-3">
                    <Datepicker id="interval_date" class="w-full justify-self-end self-end" v-model="date" vertical
                            :presetRanges="this.presetRanges" range @update:modelValue="methodUpdateDate" :clearable="false"
                            :format="this.methodFormatDatetoDatepicker"></Datepicker>
                </div>

                <div class="flex flex-col gap-2 text-gray-light px-3 w-full" v-if="this.data.solutions_type && this.data.solutions_type == 7">

                    <div class="w-full grid grid-cols-6 gap-2">

                        <div class="baltoro-container px-3 col-span-3">

                            <div class="w-full flex justify-between items-baseline py-3">
                                <i class="DK-ruido" style="font-size:1rem; color:#FFFAFA;"></i>
                                <div class="flex gap-1 whitespace-nowrap text-md">
                                    <p>{{ this.data.data[22][0] }}</p>
                                    <p>{{ this.data.units[22]}} </p>
                                </div>
                            </div>

                            <div class="w-full border-gray-semimedium border-t"></div>

                            <div class="w-full flex flex-col justify-end items-end py-2">
                                <p class="text-xs">LAeqT (1 min)</p>
                                <p class="text-xs">{{ this.limitDates(this.data.dates[0], 1) }}</p>
                            </div>

                        </div>

                        <div class="baltoro-container px-3 col-span-3">

                            <div class="w-full flex justify-between items-baseline py-3">
                                <i class="DK-ruido" style="font-size:1rem; color:#FFFAFA;"></i>
                                <div class="flex gap-1 whitespace-nowrap text-md">
                                    <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LAeq1H')}} </p>
                                    <p> {{this.data.units[22]}} </p>
                                </div>
                            </div>

                            <div class="w-full border-gray-semimedium border-t"></div>

                            <div class="w-full flex flex-col justify-end items-end py-2">
                                <p class="text-xs">LAeqT (60 min)</p>
                                <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                            </div>

                        </div>

                        <div class="col-span-2 baltoro-container px-3">

                            <div class="w-full flex justify-between items-baseline py-3">
                                <i class="DK-ruido" style="font-size:1rem; color:#FFFAFA;"></i>
                                <div class="flex gap-1 whitespace-nowrap text-md">
                                    <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LA10')}} </p>
                                    <p> {{this.data.units[22]}} </p>
                                </div>
                            </div>

                            <div class="w-full border-gray-semimedium border-t"></div>

                            <div class="w-full flex flex-col justify-end items-end py-2">
                                <p class="text-xs">LA10 (60 min)</p>
                                <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                            </div>

                        </div>

                        <div class="col-span-2 baltoro-container px-3">

                            <div class="w-full flex justify-between items-baseline py-3">
                                <i class="DK-ruido" style="font-size:1rem; color:#FFFAFA;"></i>
                                <div class="flex gap-1 whitespace-nowrap text-md">
                                    <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LA50')}} </p>
                                    <p> {{this.data.units[22]}} </p>
                                </div>
                            </div>

                            <div class="w-full border-gray-semimedium border-t"></div>

                            <div class="w-full flex flex-col justify-end items-end py-2">
                                <p class="text-xs">LA50 (60 min)</p>
                                <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                            </div>

                        </div>

                        <div class="col-span-2 baltoro-container px-3">

                            <div class="w-full flex justify-between items-baseline py-3">
                                <i class="DK-ruido" style="font-size:1rem; color:#FFFAFA;"></i>
                                <div class="flex gap-1 whitespace-nowrap text-md">
                                    <p> {{this.getNoiseData(this.data.dates[0], 'hourlyLevels', 'LA90')}} </p>
                                    <p> {{this.data.units[22]}} </p>
                                </div>
                            </div>

                            <div class="w-full border-gray-semimedium border-t"></div>

                            <div class="w-full flex flex-col justify-end items-end py-2">
                                <p class="text-xs">LA90 (60 min)</p>
                                <p class="text-xs">{{this.limitDates(this.data.dates[0], 60) }}</p>
                            </div>

                        </div>

                    </div>

                    <div class="w-full grid grid-cols-8 place-items-center gap-1 baltoro-container px-2 py-1">

                        <div></div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Mon')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(1).substring(5,10) }})</p>
                        </div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Tue')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(2).substring(5,10) }})</p>
                        </div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Wed')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(3).substring(5,10) }})</p>
                        </div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Thu')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(4).substring(5,10) }})</p>
                        </div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Fri')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(5).substring(5,10) }})</p>
                        </div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Sat')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(6).substring(5,10) }})</p>
                        </div>
                        <div class="flex flex-col items-center justify-center">
                            <p class="text-xs w-full text-center text-cyan-text">{{this.$t('general.Sun')}}</p>
                            <p class="text-xs w-full text-center whitespace-nowrap">({{ this.getDateString(0).substring(5,10) }})</p>
                        </div>

                        <p class="w-full text-center text-gray-unlight text-sm">Lday</p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(1), 'Ld')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(2), 'Ld')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(3), 'Ld')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(4), 'Ld')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(5), 'Ld')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(6), 'Ld')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(0), 'Ld')}} </p>

                        <p class="w-full text-center text-gray-unlight text-sm">Leve</p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(1), 'Le')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(2), 'Le')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(3), 'Le')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(4), 'Le')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(5), 'Le')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(6), 'Le')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(0), 'Le')}} </p>

                        <p class="w-full text-center text-gray-unlight text-sm">Lnig</p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(1), 'Ln')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(2), 'Ln')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(3), 'Ln')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(4), 'Ln')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(5), 'Ln')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(6), 'Ln')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(0), 'Ln')}} </p>

                        <p class="w-full text-center text-gray-unlight text-sm">Lden</p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(1), 'Lden')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(2), 'Lden')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(3), 'Lden')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(4), 'Lden')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(5), 'Lden')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(6), 'Lden')}} </p>
                        <p class="w-full text-center text-sm"> {{this.getNoiseData(this.getDateString(0), 'Lden')}} </p>

                    </div>

                </div>

                <div class="w-full flex overflow-x-auto snap-mandatory snap-x gap-3 px-3 pb-1">

                    <div  v-for="(chart, index) in this.Charts" :key="index" v-bind:class="{ 'hidden': chart == undefined }"
                        class="snap-center shrink-0 flex flex-col snap-always gap-2 w-full">

                        <div class="p-2 baltoro-container w-full" >
                            <p class="text-white-snow font-light text-lg mb-2 whitespace-nowrap w-full text-center">
                                {{ (chart != undefined) ? this.getName(chart) :''}}
                            </p>
                            <div class="cursor-crosshair" v-bind:class="{ 'h-[12rem]': this.data.type_name != 'PARK', 'h-[5rem]': this.data.type_name == 'PARK' }" :id="'id_chart' + index"></div>
                        </div>

                        <div class="p-2 baltoro-container w-full flex flex-row justify-between items-center" v-if="this.data.type_name == 'MODBUS' || this.data.type_name == 'PULSES' || this.data.type_name == 'MBUS'">
                            <p class="text-cyan-text font-light text-md whitespace-nowrap w-full text-center">{{ this.$t("water." + "maxAmpLang") }}</p>
                            <div class="w-full inline-flex justify-center items-end text-gray-light">
                                <p class="text-2xl font-sevensegment">
                                    {{ this.max_amplitude[index] }}
                                </p>
                                <p class="pb-0.5"> &nbsp;{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="grid text-center baltoro-container text-gray-light">

                            <p v-if="this.data.type_name == 'MODBUS' || this.data.type_name == 'PULSES' || this.data.type_name == 'MBUS'" class="col-span-2 text-cyan-text font-light text-md whitespace-nowrap w-full text-center py-1">{{ this.$t("general." + "Summary statistics") }}</p>
                            <p v-else-if="this.data.type_name == 'PARK'" class="col-span-2 text-cyan-text font-light text-md whitespace-nowrap w-full text-center py-1">{{ this.$t("general." + "Parking time statistics") }}</p>

                            <div class="col-span-2 contents">
                                <p class="self-center">{{ this.$t("general." + "Max") }}</p>
                                <div class="inline-flex justify-center items-end">
                                    <p class="text-2xl font-sevensegment self-center">
                                        {{ this.max[index] }}
                                    </p>
                                    <p class="pb-1"> &nbsp;{{ (chart != undefined) ? chart.unit : '' }}</p>
                                </div>
                            </div>

                            <div class="col-span-2 contents">
                                <p class="self-center">{{ this.$t("general." + "Avg") }}</p>
                                <div class="inline-flex justify-center items-end">
                                    <p class="text-2xl font-sevensegment self-center">
                                        {{ this.avg[index] }}
                                    </p>
                                    <p class="pb-1"> &nbsp;{{ (chart != undefined) ? chart.unit : '' }}</p>
                                </div>
                            </div>

                            <div class="col-span-2 contents">
                                <p class="self-center">{{ this.$t("general." + "Min") }}</p>
                                <div class="inline-flex justify-center items-end">
                                    <p class="text-2xl font-sevensegment self-center">
                                        {{ this.min[index] }}
                                    </p>
                                    <p class="pb-1"> &nbsp;{{ (chart != undefined) ? chart.unit : '' }}</p>
                                </div>
                            </div>

                        </div>

                        <div class="grid text-center baltoro-container text-gray-light relative" v-if="this.data.type_name == 'PARK'">
                            <div class="content-between m-0 cursor-default text-sm absolute top-0 left-0 right-0 z-50 px-2">
                                <div class="flex justify-between">

                                    <p v-if="this.data.data[0]['lecture_value'] == 1" class="service-label bg-Taken-spot text-bluerarro mt-2 invisible">
                                        {{ this.$t("general.Taken-spot") }}
                                    </p>

                                    <p v-else-if="this.data.data[0]['lecture_value'] == 0" class="service-label bg-Available-spot text-bluerarro mt-2 invisible">
                                        {{ this.$t("general.Available-spot") }}
                                    </p>

                                    <div class="flex justify-center items-center">
                                        <i v-if="this.data.data[0]['RSSI'] <= -110" class="DK-signal-01"
                                            style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else-if="this.data.data[0]['RSSI'] <= -100" class="DK-signal-02"
                                            style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else-if="this.data.data[0]['RSSI'] <= -85" class="DK-signal-03"
                                            style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else-if="this.data.data[0]['RSSI'] <= -70" class="DK-signal-04"
                                            style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else class="DK-signal-05"
                                            style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:18%"></i>

                                        <i v-if="this.data.data[0]['Battery Level'] <= 2.9"
                                            class="DK-Batt-25 animate-pulse"
                                            style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else-if="this.data.data[0]['Battery Level'] <= 3.1"
                                            class="DK-Batt-50"
                                            style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else-if="this.data.data[0]['Battery Level'] <= 3.3"
                                            class="DK-Batt-75"
                                            style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                        <i v-else class="DK-Batt-100"
                                            style="font-size:1.4rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:18%"></i>
                                    </div>
                                </div>
                            </div>

                            <div class="h-[15rem] w-full rounded-t-md mb-4" id="map"  v-on:click="this.openMapconfig()"/>

                            <JetModal :show="this.Mapconfig" :closeable="false">

                                <div class="w-full h-max flex flex-row justify-between items-center px-4 py-2 ">
                                    <div class="w-full flex flex-col justify-start">
                                        <div class="flex flex-row justify-start">
                                            <p class="text-white-snow font-light text-2xl text-left">{{ this.$t('general.Edit location') }}</p>
                                        </div>
                                    </div>
                                    <div class="flex justify-end gap-2 h-full items-start">
                                        <div class="flex items-center justify-center" v-on:click="this.CancelMap()">
                                            <i class="fa-solid fa-xmark" style="font-size:1rem; color:rgba(200, 200, 200, 0.9); "></i>
                                        </div>
                                    </div>
                                </div>

                                <div class="h-[22.5rem] w-full" id="mapModal"/>

                                <div class="flex justify-between items-center gap-x-[2.5rem] p-2 w-[90vw]">

                                    <div class="grid grid-cols-2 w-[10rem] text-sm font-light">
                                        <p class="text-left w-fit text-cyan-text">{{ this.$t("general." + "Latitude") }}:</p>
                                        <DK_Input id="id_lat" type="text" width="6rem" :value="parseFloat(this.data.device.GPS_lat)" :border="false" textPosition="center" v-on:keyup="updateLocation()"/>
                                        <p class="text-left w-fit text-cyan-text">{{ this.$t("general." + "Longitude") }}:</p>
                                        <DK_Input id="id_lng" type="text" width="6rem" :value="parseFloat(this.data.device.GPS_lng)" :border="false" textPosition="center" v-on:keyup="updateLocation()"/>
                                    </div>


                                    <div class="flex justify-end items-center">
                                        <div class="flex gap-2">

                                            <DK_Button color="blue" :fixedSize="true" v-on:click="this.SaveMap()">
                                                {{ this.$t("general.Save") }}
                                            </DK_Button>

                                        </div>
                                    </div>
                                </div>

                            </JetModal>

                            <div class="grid px-2 mb-2">
                                <div class="flex row-span-1 justify-between text-sm text-gray-light font-light gap-12">
                                    <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Name") }}</p>
                                    <DK_Input id="id_name" type="text" :border="false" width="full" :value="this.data.device.name" textPosition="right"></DK_Input>
                                </div>
                                <div class="flex row-span-1 justify-between text-sm text-gray-light font-light gap-12">
                                    <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "GatewayID") }}</p>
                                    <DK_Input type="text" :border="false" width="full" :value="this.data.device.id" textPosition="right" :readonly="true"></DK_Input>
                                </div>
                                <div class="flex row-span-1 justify-between text-sm text-gray-light font-light gap-12">
                                    <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Model") }}</p>
                                    <DK_Input type="text" :border="false" width="full" :value="this.data.device.type_name" textPosition="right" :readonly="true"></DK_Input>
                                </div>
                                <div class="flex row-span-1 justify-between text-sm text-gray-light font-light gap-12">
                                    <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Latitude") }}</p>
                                    <DK_Input id="id_lat" type="text" :border="false" width="full" :value="parseFloat(this.data.device.GPS_lat).toFixed(6)" textPosition="right"></DK_Input>
                                </div>
                                <div class="flex row-span-1 justify-between text-sm text-gray-light font-light gap-12">
                                    <p class="text-left text-cyan-text whitespace-nowrap">{{ this.$t("general." + "Longitude") }}</p>
                                    <DK_Input id="id_lng" type="text" :border="false" width="full" :value="parseFloat(this.data.device.GPS_lng).toFixed(6)" textPosition="right"></DK_Input>
                                </div>
                            </div>

                            <div class="flex justify-center items-center mb-2 gap-2">

                                <DK_Button color="blue" :fixedSize="true" v-on:click="this.Save_update()">
                                    {{ this.$t("general.Save") }}
                                </DK_Button>

                                <DK_Button color="blue" :fixedSize="true" v-on:click="this.Cancel_update()">
                                    {{ this.$t("general.Cancel") }}
                                </DK_Button>

                            </div>

                        </div>

                        <!--<div class="p-3 baltoro-container" v-if="this.data.type_name == 'MODBUS' || this.data.type_name == 'PULSES'">

                            <table id="basicTable" class="row-border">
                                <thead class="bg-transparent text-gray-light">
                                    <tr>
                                        <th>{{ this.$t("general." + "Date") }}</th>
                                        <th>{{ (chart != undefined) ? this.getName(chart) :''}} {{ (chart != undefined) ? chart.unit : '' }}</th>
                                    </tr>
                                </thead>
                                <tbody class="text-center">

                                </tbody>
                            </table>
                        </div>-->

                        <!--<div class="2xl:col-span-4 col-span-5 pt-8 px-10 baltoro-container">

                            <table id="basicTable" class="row-border">
                                <thead class="bg-transparent text-gray-light">
                                    <tr>
                                        <th>{{ this.$t("general." + "Date start") }}</th>
                                        <th>{{ this.$t("general." + "Date end") }}</th>
                                        <th>{{ this.$t("general." + "Time_mode") }}</th>
                                        <th>{{ this.$t("general." + "Parkingspace") }}</th>
                                    </tr>
                                </thead>
                                <tbody class="text-center">

                                </tbody>
                            </table>

                        </div>-->

                    </div>
                </div>

                <div class="w-full px-3 pb-1">
                    <div class="py-2 px-4 baltoro-container overflow-x-scroll w-[40rem]">

                        <table id="basicTable" class="row-border" v-if="this.data.type_name == 'PARK'">
                            <thead class="bg-transparent text-gray-light">
                                <tr>
                                    <th>{{ this.$t("general." + "Date start") }}</th>
                                    <th>{{ this.$t("general." + "Date end") }}</th>
                                    <th>{{ this.$t("general." + "Time_mode") }}</th>
                                    <th>{{ this.$t("general." + "Parkingspace") }}</th>
                                </tr>
                            </thead>
                            <tbody class="text-center">

                            </tbody>
                        </table>

                        <table id="basicTable" class="row-border" v-else>
                            <thead class="bg-transparent text-gray-light">
                                <tr>
                                    <th>{{ this.$t("general." + "Date") }}</th>
                                    <th v-for="(data, index) in this.data.data" :key="index">
                                        <!--{{ this.data.names[index] + ' (' + this.data.units[index] + ')' }}-->
                                        <a v-if="this.data.units[index] != ''">
                                            {{ this.data.names[index] + ' (' + this.data.units[index]+ ')' }}
                                        </a>
                                        <a v-else>
                                            {{ this.data.names[index] }}
                                        </a>
                                    </th>
                                </tr>
                            </thead>
                            <tbody class="text-center">
                            </tbody>
                        </table>

                    </div>
                </div>
            </div>
        </div>
    </AppLayoutCity>
</template>

<script>
export default {
    props: ['data', 'user'],

    data() {
        return {
            screen_width: 0,
            Back: localStorage.getItem('Back'),
            presetRanges: [
                { label: this.$t("general." + "Today"), range: [startOfDay(new Date()), new Date()] },
                { label: this.$t("general." + "Yesterday"), range: [startOfDay(subDays(new Date(), 1)), endOfDay(subDays(new Date(), 1))] },
                { label: this.$t("general." + "Last 7 Days"), range: [new Date().setDate(new Date().getDate() - 7), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 14 Days"), range: [new Date().setDate(new Date().getDate() - 14), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 30 Days"), range: [new Date().setDate(new Date().getDate() - 30), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "This month"), range: [startOfMonth(new Date()), new Date()] },
                { label: this.$t("general." + "Last month"), range: [startOfMonth(subMonths(new Date(), 1)), endOfMonth(subMonths(new Date(), 1))] }
            ],

            formatDate: (date) => {

                date = new Date(date);

                return (
                    date.getUTCFullYear() + "-" +
                    ((date.getUTCMonth() + 1 + '').length < 2 ? "0" + (date.getUTCMonth() + 1) : (date.getUTCMonth() + 1)) + "-" +
                    ((date.getUTCDate() + '').length < 2 ? "0" + date.getUTCDate() : date.getUTCDate()) + " " +
                    ((date.getUTCHours() + '').length < 2 ? "0" + date.getUTCHours() : date.getUTCHours()) + ":" +
                    ((date.getUTCMinutes() + '').length < 2 ? "0" + date.getUTCMinutes() : date.getUTCMinutes()) + ":" +
                    ((date.getUTCSeconds() + '').length < 2 ? "0" + date.getUTCSeconds() : date.getUTCSeconds())
                );
            },

            formatDateLocal: (date, selector, selector2) => {

                if(selector){
                    // Fecha actual
                    var currentDate = date;

                    // Obtener el desfase horario actual de la zona horaria de la fecha actual
                    var currentOffset = -1 * currentDate.getTimezoneOffset() / 60;

                    // Calcular la diferencia de desfase horario
                    var offsetDiff = this.data.device.hoursOffset - currentOffset;

                    // Convertir la fecha actual a tiempo en milisegundos
                    var currentTime = currentDate.getTime();

                    // Calcular el desfase horario en milisegundos
                    var offsetMillis = offsetDiff * 60 * 60 * 1000;

                    // Añadir el desfase horario a la fecha actual
                    var newDate = new Date(currentTime + offsetMillis);

                    // Reemplazar la fecha original con la nueva fecha

                    if(!selector2)
                        date = newDate;
                    else
                        return newDate;

                    return (
                        date.getFullYear() + "-" +
                        ((date.getMonth() + 1 + '').length < 2 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1)) + "-" +
                        ((date.getDate() + '').length < 2 ? "0" + date.getDate() : date.getDate()) + " " +
                        ((date.getHours() + '').length < 2 ? "0" + date.getHours() : date.getHours()) + ":" +
                        ((date.getMinutes() + '').length < 2 ? "0" + date.getMinutes() : date.getMinutes()) + ":" +
                        ((date.getSeconds() + '').length < 2 ? "0" + date.getSeconds() : date.getSeconds())
                    );

                }else{

                    date = new Date(date);

                    return (
                        date.getFullYear() + "-" +
                        ((date.getMonth() + 1 + '').length < 2 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1)) + "-" +
                        ((date.getDate() + '').length < 2 ? "0" + date.getDate() : date.getDate()) + " " +
                        ((date.getHours() + '').length < 2 ? "0" + date.getHours() : date.getHours()) + ":" +
                        ((date.getMinutes() + '').length < 2 ? "0" + date.getMinutes() : date.getMinutes()) + ":" +
                        ((date.getSeconds() + '').length < 2 ? "0" + date.getSeconds() : date.getSeconds())
                    );
                }
            },

            calcularTiempoTranscurrido: (fechaInicioStr, fechaFinStr, short) => {
                var diferenciaEnMilisegundos = 0;
                var disjoin = '';

                if(typeof fechaFinStr != 'undefined'){
                    const fechaInicio = new Date(fechaInicioStr);
                    const fechaFin = new Date(fechaFinStr);

                    diferenciaEnMilisegundos = fechaFin - fechaInicio;
                } else {
                    diferenciaEnMilisegundos = fechaInicioStr * 1000;
                }

                // Convertir la diferencia a días, horas, minutos y segundos
                const segundos = Math.floor(diferenciaEnMilisegundos / 1000);
                const minutos = Math.floor(segundos / 60);
                const horas = Math.floor(minutos / 60);
                const dias = Math.floor(horas / 24);

                // Crear un array para almacenar las partes no nulas del tiempo
                const partesTiempo = [];

                if(short) {
                    disjoin = ' ';
                    if (dias > 0) partesTiempo.push(`${dias} d`);
                    if (horas % 24 > 0) partesTiempo.push(`${horas % 24} h`);
                    if (minutos % 60 > 0) partesTiempo.push(`${minutos % 60} min`);
                    if (segundos % 60 > 0 && partesTiempo.length < 3) partesTiempo.push(`${segundos % 60} s`);
                } else {
                    disjoin = ', ';
                    if (dias > 0) partesTiempo.push(`${dias} día${dias !== 1 ? 's' : ''}`);
                    if (horas % 24 > 0) partesTiempo.push(`${horas % 24} hora${horas % 24 !== 1 ? 's' : ''}`);
                    if (minutos % 60 > 0) partesTiempo.push(`${minutos % 60} minuto${minutos % 60 !== 1 ? 's' : ''}`);
                    if (segundos % 60 > 0 && partesTiempo.length < 3) partesTiempo.push(`${segundos % 60} segundo${segundos % 60 !== 1 ? 's' : ''}`);
                }

                // Unir las partes del tiempo en un string
                var tiempoFormateado = partesTiempo.join(disjoin);

                if(diferenciaEnMilisegundos == 0) {
                    tiempoFormateado = '0 segundos';
                }

                return tiempoFormateado;

            },

            formatTime: (date_state_change, diff) => {

                let duration_ms;
                if (diff == true) {
                    duration_ms = new Date() - new Date(date_state_change);
                } else {
                    duration_ms = date_state_change * 1000;
                }

                const diff_D = Math.floor(duration_ms / (1000 * 60 * 60 * 24));
                const diff_H = Math.floor(
                    (duration_ms % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60)
                );
                const diff_M = Math.floor(
                    (duration_ms % (1000 * 60 * 60)) / (1000 * 60)
                );

                let duration = "";

                if (this.user.language == "es" && diff) {
                    duration = this.$t("general.for") + " ";
                }

                if (diff_D > 0) {
                    duration += this.$t("general.morethan") + " " + diff_D + " ";
                    if (diff_D > 1) {
                        duration += this.$t("general.days");
                    } else {
                        duration += this.$t("general.day");
                    }
                } else {
                    if (diff_H > 0) {
                        duration += diff_H;
                        if (diff_M > 0) {
                            duration += this.$t("general.h");
                        } else if (diff_H > 1) {
                            duration += " " + this.$t("general.hours");
                        } else {
                            duration += " " + this.$t("general.hour");
                        }

                        if (diff_M != 0) {
                            duration += " " + this.$t("general.and") + " ";
                        }
                    }

                    if (diff_M > 0) {
                        duration += diff_M;
                        if (diff_H != 0 && diff_M > 1) {
                            duration += this.$t("general.mins");
                        } else if (diff_H != 0 && diff_M == 1) {
                            duration += this.$t("general.min");
                        } else if (diff_M > 1) {
                            duration += " " + this.$t("general.minutes");
                        } else {
                            duration += " " + this.$t("general.minute");
                        }
                    }
                }

                if (diff_H == 0 && diff_H == 0 && diff_M == 0) {
                    duration = "0 " + this.$t("general.minutes");
                }

                if (this.user.language == "en" && diff) {
                    duration += " " + this.$t("general.ago");
                }

                return duration;
            },

            colors: ["#05fff3", "#FF29D9", "#FFC100", "#6DFF33", "#F53A1F"],
            date: undefined,
            default_Chart: undefined,
            Charts: Array.from({ length: Object.keys(this.data.names).length }, () => undefined),
            Highcharts: require('highcharts'),
            VMoptions: new Array(),
            VMmodel: new Array(),
            max: new Array(),
            min: new Array(),
            avg: new Array(),
            max_amplitude: new Array(),
            charts_size: new Array(),
            device_update: new Object(),
            Mapconfig: false,
            charts_mode: true, //false son serparados y true son juntos por unidad

        }
    },

    beforeCreate() {
        this.$i18n.locale = this.user.language
    },

    beforeMount() {

        this.screen_width = window.screen.width;

        this.Charts.forEach((chart, index) => {
            this.VMoptions[index] = new Array();
        })

        this.VMoptions[0] = new Array();

        if(this.data.type_name != null){

            if(typeof this.data.dates == 'undefined'){
                this.data.dates = [new Date()];
            }

            this.date = [
                new Date(this.data.dates[this.data.dates.length - 1]),
                this.data.type_name == "PARK"? new Date() : new Date(this.data.dates[0])
            ];

        }else{
            this.date = [
                new Date((new Date()).setDate((new Date()).getDate() - 1)),
                new Date()
            ]
        }

        this.device_update = {
            name: this.data.device.name,
            GPS_lat: parseFloat(this.data.device.GPS_lat).toFixed(6),
            GPS_lng: parseFloat(this.data.device.GPS_lng).toFixed(6)
        }

    },

    mounted() {

        window.emitter.emit('changeLoaderStatus', false, 'orbit')

        let export_title = this.data.device.name + '_' + this.data.device.id + '_' + this.formatDate(new Date())

        //if(this.screen_width >= 768)

        this.table = $('#basicTable').DataTable({
            language: {
                url: '/assets/i18n/' + this.user.language + '.json'
            },
            order: [[0, "desc"]],
            dom: '<"row"<"col-sm-6"B><"col-sm-6">>t<"row"<"col-sm-6"i><"col-sm-6"p>><"clear">',
            ordering: true,
            pageLength: 10,
            buttons: [
                {
                    extend: 'csv',
                    text: 'CSV',
                    title: export_title,
                    className: 'btn btn-brand',
                    exportOptions: {
                        columns: ':visible'
                    }
                },
                {
                    extend: 'excel',
                    text: 'Excel',
                    title: export_title,
                    className: 'btn btn-brand',
                    exportOptions: {
                        columns: ':visible'
                    }
                },
                {
                    extend: 'print',
                    text: 'Print',
                    title: export_title,
                    className: 'btn btn-brand',
                    exportOptions: {
                        columns: ':visible'
                    }
                },
            ]
        });

        this.rechargePage();

    },

    methods: {

        rechargePage() {

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            let formatDate = this.formatDate;

            const Highcharts = require('highcharts');

            this.charts_size = new Array();

            let Charts = Array.from({ length: Object.keys(this.data.names).length }, () => undefined);

            let default_Chart = {
                chart: {
                    zoomType: 'x',
                    backgroundColor: 'transparent',
                    spacingTop: 20,
                    animation: {
                        duration: 150
                    },
                    resetZoomButton: {
                        position: {
                            align: 'left',
                            verticalAlign: 'top',
                            x: 0,
                            y: 2.5
                        },
                        theme: {
                            r: 5.6,
                        }
                    }
                },
                accessibility: {
                    enabled: false
                },
                title: {
                    text: ''
                },
                xAxis: {
                    lineColor: '#647F8C',
                    visible: true,
                    labels: {
                        style: {
                            color: "rgba(200, 200, 200, 0.9)",
                            opacity: 0.7,
                        },
                    },
                    type: "datetime",
                    /*dateTimeLabelFormats: {
                        //millisecond: '%H:%M:%S.%L',
                        //second: '%H:%M:%S',
                        //minute: '%H:%M',
                        hour: '%H:%M',
                        day: '%m-%d',
                        week: '%m-%d',
                        month: '%Y-%m',
                        year: '%Y'
                    },*/
                    time: {
                        timezoneOffset: 0, // Configura la zona horaria actual
                        useUTC: true // Utiliza hora local
                    }
                },
                yAxis: new Array(),
                exporting: {
                    enabled: false
                },
                credits: {
                    enabled: false
                },
                legend: {
                    enabled: true,
                    itemStyle: {
                        color: 'rgba(200, 200, 200, 0.9)',
                        opacity: 0.75,
                        fontWeight: 'light'
                    },
                    itemHoverStyle: {
                        color: 'rgba(200, 200, 200, 0.9)',
                        opacity: 1,
                        fontWeight: 'light'
                    }
                },
                tooltip: {
                    shared: true,
                    split: false,
                    backgroundColor: 'rgba(7,82,103,0.95)',
                    borderRadius: 12,
                    borderWidth: 1,
                    borderColor: 'rgba(250, 250, 250, 0.65)',
                    padding: 6,
                    formatter: function () {
                        var s = '<b>' + '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' + formatDate(this.x) + '</p><br/>';

                        $.each(this.points, function () {
                            s += '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.series.name + ' </p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.y +
                                ' ' + this.series.userOptions.tooltip.valueSuffix + '</p><br/>';
                        });

                        return s;
                    },
                },
            };

            if(this.data.type_name == null){



            }else if (this.data.type_name == "MODBUS" || this.data.type_name == "PULSES" || this.data.type_name == "MBUS") {

                let sensor_num;

                if (this.data.data) {
                    sensor_num = Object.keys(this.data.data).length
                } else {
                    sensor_num = 1;
                }

                let units = new Array(sensor_num);
                let names = new Array(sensor_num);
                let data = new Array(sensor_num);
                let dates;

                for (let sensor in this.data.data) {

                    dates = new Array();
                    data[sensor] = new Array();

                    names[sensor] = this.data.names[sensor];
                    units[sensor] = this.data.units[sensor];

                    let newChart = true;

                    Charts.forEach((chart, index) => {
                        if (chart != undefined) {
                            if (
                                chart.unit == units[sensor] &&
                                this.charts_mode &&
                                (
                                    units[sensor] != '' ||
                                    this.nameConcordance(names, sensor, Charts[index].sensors, units[sensor])
                                )
                            ) {
                                newChart = false;
                                Charts[index].sensors.push(sensor)
                            }
                        }
                    })

                    if (newChart == true) {

                        let index = Charts.indexOf(undefined);

                        Charts[index] = {
                            unit: units[sensor],
                            highcharts: default_Chart,
                            sensors: [sensor],
                            id: 'id_chart' + index,
                        }
                    }

                    this.data.dates.forEach((date, index) => {
                        dates.push(new Date(date.replace(/-/g, '/') + ' UTC').getTime());
                        data[sensor].push(parseFloat(this.data.data[sensor][index]));
                    })

                    //this.data.data[sensor].forEach((value, index) => {
                    //    data[sensor].push(parseFloat(value));
                    //})

                }

                for (let i_chart = 0; i_chart < Charts.length; i_chart++) {

                    if (Charts[i_chart] != undefined) {

                        Charts[i_chart].highcharts.yAxis = (
                            {
                                id: 0,
                                gridLineColor: '#647F8C',
                                title: {
                                    text: ''
                                },
                                labels: {
                                    formatter: function () {
                                        return this.pos + ' ' + Charts[i_chart].unit
                                    },
                                    style: {
                                        color: 'rgba(200, 200, 200, 0.9)',
                                        opacity: 0.7,
                                    }
                                },
                                visible: true,
                            }
                        );

                        Charts[i_chart].highcharts.series = new Array();

                        data.forEach((sensor, index) => {

                            if (Charts[i_chart].sensors.includes(index.toString())) {

                                let highcharts_data = new Array(sensor.length);

                                sensor.forEach((measure, i) => {
                                    if(!isNaN(measure)) {
                                        highcharts_data.push([dates[i], measure])
                                    }
                                })

                                Charts[i_chart].highcharts.tooltip.valueSuffix = units[index];

                                Charts[i_chart].highcharts.series.push({
                                    name: names[index],
                                    type: 'spline',
                                    yAxis: 0,
                                    data: highcharts_data.filter(point => point !== undefined && point !== null).reverse(),
                                    color: this.colors[(Charts[i_chart].sensors.indexOf(index.toString()) % 5)],
                                    lineWidth: 1.5,
                                    zIndex: 1,
                                    marker: {
                                        enabled: false
                                    },
                                    tooltip: {
                                        valueSuffix: units[index]
                                    }
                                });
                            }
                        })

                        Charts[i_chart].chart = Highcharts.chart(Charts[i_chart].id, Charts[i_chart].highcharts);
                    }

                }

                setTimeout(() => {
                    for (let i_chart = 0; i_chart < Charts.length; i_chart++) {
                        if (Charts[i_chart] != undefined && document.getElementById(Charts[i_chart].id)) {
                                Charts[i_chart].chart.reflow();
                        }
                    }
                }, 0);

                this.Charts = Charts;

                this.table.clear();

                if(typeof this.data.data != 'undefined'){

                    this.data.dates.forEach((date, index) => {

                        let temp_row = ['<td>' + this.formatDate(date.replace(/-/g, '/') + ' UTC') + '</td>'];

                        for (let sensor in this.data.data) {

                            if (typeof this.data.data[sensor][index] !== 'undefined') {
                                temp_row.push('<td>' + parseFloat(this.data.data[sensor][index]) + '</td>')
                            }else{
                                temp_row.push('<td></td>')
                            }

                        }

                        if (temp_row.length > 1)
                            this.table.row.add(temp_row).draw(false);
                    })
                } else {
                    this.table.draw(false)
                }

                this.Charts.forEach((chart, index) => {
                    if (chart != undefined) {

                        this.VMoptions[index] = new Array();

                        for (let id in chart.sensors) {

                            this.VMoptions[index].push({
                                id: chart.sensors[id],
                                name: this.data.names[chart.sensors[id]],
                            })
                        }

                        this.VMmodel[index] = this.VMoptions[index][0]

                        if (parseFloat(this.data.stats[chart.sensors[0]].max_amplitude) <= 0.1 && chart.sensors.length <= 1) {
                            this.charts_size.push(index);
                        }

                        this.max[index] = parseFloat(this.data.stats[chart.sensors[0]].max)
                        this.min[index] = parseFloat(this.data.stats[chart.sensors[0]].min)
                        this.max_amplitude[index] = parseFloat(this.data.stats[chart.sensors[0]].max_amplitude)
                        this.avg[index] = parseFloat(this.data.stats[chart.sensors[0]].avg)

                    }
                })

            } else if (this.data.type_name == "PARK") {

                let sensor_num;

                if (this.data.data) {
                    sensor_num = Object.keys(this.data.data).length
                } else {
                    sensor_num = 1;
                }

                let units = new Array(sensor_num);
                let names = new Array(sensor_num);
                let data = new Array(sensor_num);
                let dates;

                for (let sensor in this.data.data) {

                    dates = new Array();
                    data[sensor] = new Array();

                    names[sensor] = this.data.names[sensor];
                    units[sensor] = this.data.units[sensor];

                    let newChart = true;

                    Charts.forEach((chart, index) => {
                        if (chart != undefined) {
                            if (chart.unit == units[sensor]) {
                                newChart = false;
                                Charts[index].sensors.push(sensor)
                            }
                        }
                    })

                    if (newChart == true) {

                        let index = Charts.indexOf(undefined);

                        Charts[index] = {
                            unit: units[sensor],
                            highcharts: default_Chart,
                            sensors: [sensor],
                            id: 'id_chart' + index,
                        }

                        if (sensor == '0') {
                            this.charts_size.push(index);
                        }
                    }

                    if(typeof this.data.data != 'undefined'){
                        this.data.data.forEach((register, index) => {
                            data[sensor].push(parseFloat(register['lecture_value']));
                        })
                    }


                    if(typeof this.data.dates != 'undefined'){
                        this.data.dates.forEach((date, index) => {
                            dates.push(new Date((date + ' GMT')).getTime());
                        })
                    }
                }

                for (let i_chart = 0; i_chart < Charts.length; i_chart++) {

                    if (Charts[i_chart] != undefined) {

                        if (i_chart == 0) {

                            let highcharts_data_occupied = new Array();

                            for (let i = dates.length - 1; i >= 0; i--) {
                                highcharts_data_occupied.push([dates[i], data[0][i]])

                                if(i == 0){
                                    const fechaUTC = new Date(this.formatDateLocal(this.date[1], true, true));
                                    const offsetUTC = fechaUTC.getTimezoneOffset();
                                    const epochUTC = fechaUTC.getTime() - (offsetUTC * 60 * 1000);
                                    highcharts_data_occupied.push([epochUTC, data[0][i]])
                                }
                            }

                            let highcharts_data_free = new Array(highcharts_data_occupied.length);

                            highcharts_data_occupied.forEach((measure, index) => {
                                highcharts_data_free[index] = [measure[0], 1 - measure[1]];
                            })

                            const $t = this.$t;

                            Charts[i_chart].highcharts = {
                                chart: {
                                    backgroundColor: "transparent",
                                    zoomType: 'x',
                                    animation: {
                                        duration: 150
                                    },
                                    resetZoomButton: {
                                        position: {
                                            align: 'left',
                                            verticalAlign: 'top',
                                            x: 0,
                                            y: 2.5
                                        },
                                        theme: {
                                            r: 5.6,
                                        }
                                    }
                                },
                                accessibility: {
                                    enabled: false,
                                },
                                title: {
                                    text: "",
                                },
                                xAxis: {
                                    lineColor: '#647F8C',
                                    visible: true,
                                    labels: {
                                        formatter: function () {
                                            return Highcharts.dateFormat('%H:%M', this.value);
                                        },
                                        style: {
                                            color: "rgba(200, 200, 200, 0.9)",
                                            opacity: 0.7,
                                        },
                                    },
                                    type: "datetime",
                                    /*dateTimeLabelFormats: {
                                        //millisecond: '%H:%M:%S.%L',
                                        //second: '%H:%M:%S',
                                        //minute: '%H:%M',
                                        hour: '%H:%M',
                                        day: '%m-%d',
                                        week: '%m-%d',
                                        month: '%Y-%m',
                                        year: '%Y'
                                    },*/
                                },
                                yAxis: {
                                    endOnTick: false,
                                    visible: false,
                                },
                                exporting: {
                                    enabled: false,
                                },
                                credits: {
                                    enabled: false,
                                },
                                legend: {
                                    enabled: false,
                                },
                                plotOptions: {
                                    area: {
                                        lineWidth: 0,
                                    },
                                    stacking: 'normal',
                                    series: {
                                        marker: {
                                            enabled: false,
                                            states: {
                                                hover: {
                                                    enabled: false
                                                }
                                            }
                                        }
                                    }
                                },
                                series: [
                                    {
                                        type: "area",
                                        name: '',
                                        data: highcharts_data_occupied,
                                        color: "#FF0000",
                                        zIndex: 1,
                                        step: 'left',
                                        marker: {
                                            enabled: false,
                                            symbol: null, // elige un cuadrado como marcador
                                            fillColor: null, // color de relleno del marcador
                                            lineWidth: 0, // grosor del borde del marcador
                                            lineColor: null // color del borde del marcador
                                        },
                                        tooltip: {
                                            valueSuffix: this.$t("general.Taken-spot"),
                                        },
                                    },
                                    {
                                        type: "area",
                                        name: '',
                                        data: highcharts_data_free,
                                        color: "#99FF00",
                                        zIndex: 1,
                                        step: 'left',
                                        marker: {
                                            enabled: false,
                                            symbol: null, // elige un cuadrado como marcador
                                            fillColor: null, // color de relleno del marcador
                                            lineWidth: 0, // grosor del borde del marcador
                                            lineColor: null // color del borde del marcador
                                        },
                                        tooltip: {
                                            valueSuffix: this.$t("general.Available-spot"),
                                        },
                                    },
                                ],
                                tooltip: {
                                    valueSuffix: "",
                                    shared: true,
                                    split: false,
                                    backgroundColor: "rgba(7,82,103,0.95)",
                                    borderRadius: 12,
                                    borderWidth: 1,
                                    borderColor: "rgba(250, 250, 250, 0.65)",
                                    padding: 6,

                                    formatter: function () {
                                        const date = new Date(this.x);

                                        const year = date.getUTCFullYear();
                                        const month = ('0' + (date.getUTCMonth() + 1)).slice(-2);
                                        const day = ('0' + date.getUTCDate()).slice(-2);
                                        const hours = ('0' + date.getUTCHours()).slice(-2);
                                        const minutes = ('0' + date.getUTCMinutes()).slice(-2);

                                        const formattedDateTime = `${year}-${month}-${day} ${hours}:${minutes}`;

                                        var s = '';

                                        $.each(this.points, function () {

                                            if (this.y == 1) {

                                                s +=
                                                    '<p style="font-size:16px; line-height: 16px; font-weight: 400; color:' + this.color + '"> ' +
                                                    this.series.userOptions.tooltip.valueSuffix +
                                                    "</p><br/>";
                                            }
                                        });

                                        s +=
                                            "<b>" +
                                            '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
                                            $t('general.Since') + ' ' +
                                            formattedDateTime +
                                            "</p><br/>";

                                        return s;
                                    },
                                },
                            };

                            Charts[i_chart].chart = Highcharts.chart(Charts[i_chart].id, Charts[i_chart].highcharts);

                        } else {

                            Charts[i_chart].highcharts.yAxis = (
                                {
                                    id: 0,
                                    gridLineColor: '#647F8C',
                                    title: {
                                        text: ''
                                    },
                                    labels: {
                                        formatter: function () {
                                            return this.pos + ' ' + Charts[i_chart].unit
                                        },
                                        style: {
                                            color: 'rgba(200, 200, 200, 0.9)',
                                            opacity: 0.7,
                                        }
                                    },
                                    visible: true,
                                }
                            );

                            Charts[i_chart].highcharts.series = new Array();

                            data.forEach((sensor, index) => {

                                if (Charts[i_chart].sensors.includes(index.toString())) {

                                    let highcharts_data = this.data.summary.highcharts_data;

                                    Charts[i_chart].highcharts.tooltip.valueSuffix = units[index];

                                    Charts[i_chart].highcharts.series.push({
                                        name: names[index],
                                        //type: 'spline',
                                        step: 'left',
                                        yAxis: 0,
                                        data: highcharts_data.filter(point => point !== undefined && point !== null).reverse(),
                                        color: this.colors[(Charts[i_chart].sensors.indexOf(index) % 5)],
                                        lineWidth: 1.5,
                                        zIndex: 1,
                                        marker: {
                                            enabled: false
                                        },
                                        tooltip: {
                                            valueSuffix: units[index]
                                        }
                                    });
                                }
                            })

                            Charts[i_chart].chart = Highcharts.chart(Charts[i_chart].id, Charts[i_chart].highcharts);

                        }
                    }
                }

                setTimeout(() => {
                    for (let i_chart = 0; i_chart < Charts.length; i_chart++) {
                        if (Charts[i_chart] != undefined && document.getElementById(Charts[i_chart].id)) {
                            Charts[i_chart].chart.reflow();
                        }
                    }
                }, 0);

                this.Charts = Charts;

                this.table.clear();

                if(typeof this.data.data != 'undefined'){

                    this.data.data.forEach((register, index) => {

                        let temp_row = new Array();

                        let Progress_label = (typeof this.data.dates[index - 1] != 'undefined' ? '' : ('<div class="popup-container">' +
                                '<span class="popup-text">' + this.$t('general.In progress') + '</span>' +
                                '<div class="popup">' +
                                    '<p>' + this.$t('text.In progress') + '</p>' +
                                '</div>' +
                            '</div>'));

                        let endDate = (typeof this.data.dates[index - 1] == 'undefined' ? this.formatDateLocal(this.date[1], true) : this.data.dates[index - 1]);

                        temp_row.push('<td>' + this.data.dates[index] + '</td>');
                        temp_row.push('<td>' + endDate + '</td>');
                        temp_row.push('<td>' + this.calcularTiempoTranscurrido(this.data.dates[index], endDate, false) + '</td>');

                        if (parseFloat(register['lecture_value']) != 0) {
                            temp_row.push('<td><div class="flex flex-row justify-center gap-2">' + this.$t("general.Taken-spot") + ' ' + Progress_label +'</div></td>');
                        } else {
                            temp_row.push('<td><div class="flex flex-row justify-center gap-2">' + this.$t("general.Available-spot") + ' ' + Progress_label +'</div></td>');
                        }

                        if (temp_row.length > 1)
                            this.table.row.add(temp_row).draw(false);
                    })
                } else {
                    this.table.draw(false)
                }

                this.Charts.forEach((chart, index) => {
                    if (chart != undefined) {

                        this.VMoptions[index] = new Array();

                        for (let id in chart.sensors) {

                            this.VMoptions[index].push({
                                id: chart.sensors[id],
                                name: this.data.names[chart.sensors[id]],
                            })
                        }

                        this.VMmodel[index] = this.VMoptions[index][0]

                        this.max[index] = this.calcularTiempoTranscurrido(this.data.stats.max, undefined, true);
                        this.min[index] = this.calcularTiempoTranscurrido(this.data.stats.min, undefined, true);
                        this.avg[index] = this.calcularTiempoTranscurrido(this.data.stats.avg, undefined, true);
                    }

                })

                mapboxgl.accessToken = 'pk.eyJ1IjoiZGF0YWtvcnVtLWRldmVsb3AiLCJhIjoiY2w0ODhqOXc2MDR1MjNjbzJ2YnZkdjNhYSJ9.pVPMnmkwOuF4DonlbNkkEw';

                const map = new mapboxgl.Map({
                    attributionControl: false,
                    container: "map",
                    style: 'mapbox://styles/datakorum-develop/cl4cu370w000v15tal53sxn9v',
                    center: [this.data.device.GPS_lng, this.data.device.GPS_lat],
                    zoom: 15,
                    pitch: 67.5,
                    bearing: 0,
                    interactive: false
                });

                const img = document.createElement('img');

                if(typeof this.data.data != 'undefined' && parseFloat(this.data.data[0]['lecture_value']) == 0){
                    this.str_img = "../assets/images/markers/marker_park_aval.png";
                } else if(typeof this.data.data != 'undefined' && parseFloat(this.data.data[0]['lecture_value']) == 1) {
                    this.str_img = "../assets/images/markers/marker_park_take.png";
                } else {
                    this.str_img = "../assets/images/markers/marker_park.png";
                }

                img.src = this.str_img;

                const markerElement = document.createElement('div');
                markerElement.appendChild(img);

                const marker = new mapboxgl.Marker({ element: markerElement })
                    .setLngLat([this.data.device.GPS_lng, this.data.device.GPS_lat])
                    .addTo(map);

            }

            window.emitter.emit('changeLoaderStatus', false, 'orbit')

        },

        setDateIndex(week_day){

            let date = new Date(this.data.daily.dates[0]);
            let week_num = date.getDay();
            let count = 0;

            while(week_num != week_day){
                date.setDate(date.getDate() - 1);
                week_num = date.getDay();
                count ++;
            }

            return count;
        },

        getDateString(week_day){

            let date = new Date(this.data.dates[0]);
            let week_num = date.getDay();

            while(week_num != week_day){
                date.setDate(date.getDate() - 1);
                week_num = date.getDay();
            }

            let year = date.getFullYear();
            let month = String(date.getMonth() + 1).padStart(2, '0'); // Months are zero-based
            let day = String(date.getDate()).padStart(2, '0');

            let formattedDate = `${year}-${month}-${day}`;

            return formattedDate;
        },

        limitDates(date, mins){

            let start_date, end_date;

            if(mins == 60){
                var fecha = new Date(date);
                fecha.setUTCMinutes(0)
            }else if(mins == 10){
                var fecha = new Date(date);
                fecha.setUTCMinutes(fecha.getUTCMinutes() - (fecha.getUTCMinutes() % 10))
            }else{
                var fecha = new Date(date);
            }

            end_date = ('0' + fecha.getHours()).slice(-2) + ':' + ('0' + fecha.getUTCMinutes()).slice(-2);

            fecha.setUTCMinutes(fecha.getUTCMinutes() - mins);

            start_date = ('0' + fecha.getHours()).slice(-2) + ':' + ('0' + fecha.getUTCMinutes()).slice(-2);

            return start_date + ' ' + this.$t('general.To') + ' ' + end_date;
        },

        getNoiseData(date, measure1, measure2){

            let data;

            if(date != undefined){

                let day = date.substring(0, 10);
                let hours = date.substring(11, 13);

                var index = this.data.daily.dates.indexOf(day);

                if (index !== -1) {

                    if(measure1 == 'hourlyLevels' && measure2 != undefined){
                        data = this.data.daily.data[measure1][index][hours][measure2]
                    }else if(measure1 != undefined ){
                        data = this.data.daily.data[measure1][index]
                    }else{
                        data = '-';
                    }

                } else {

                    data = '-';

                }

            } else {

                data = '-';

            }

            return data;

        },

        openMapconfig() {
            this.Mapconfig = true;

            setTimeout(() => {

                // creating map
                this.mapModal = new mapboxgl.Map({
                attributionControl: false,
                container: "mapModal",
                style: 'mapbox://styles/datakorum-develop/cl4cu370w000v15tal53sxn9v',
                center: [this.data.device.GPS_lng, this.data.device.GPS_lat],
                zoom: 15,
                pitch: 50,
                bearing: 0,
                });

                this.markers = [];

                let img = document.createElement('img');
                img.src = this.str_img;

                let markerElement = document.createElement('div')
                markerElement.appendChild(img)

                this.markers.push(new mapboxgl.Marker({ element: markerElement })
                    .setLngLat([this.data.device.GPS_lng, this.data.device.GPS_lat])
                    .addTo(this.mapModal));

                this.mapModal.on("click", (event) => {

                    this.data.device.GPS_lat = parseFloat(event.lngLat.lat).toFixed(6)
                    this.data.device.GPS_lng = parseFloat(event.lngLat.lng).toFixed(6)

                    this.mapModal.flyTo({
                        center: [this.data.device.GPS_lng, this.data.device.GPS_lat],
                        essential: true,
                    });

                    // eliminar todos los marcadores existentes
                    this.markers.forEach(marker => marker.remove());
                    this.markers = [];

                    this.markers.push(new mapboxgl.Marker({ element: markerElement })
                        .setLngLat([this.data.device.GPS_lng, this.data.device.GPS_lat])
                        .addTo(this.mapModal));

                });

            }, 0);

        },

        updateLocation(){

            this.data.device.GPS_lat = document.getElementById('id_map_lat').value
            this.data.device.GPS_lng = document.getElementById('id_map_lng').value

            this.mapModal.flyTo({
                center: [this.data.device.GPS_lng, this.data.device.GPS_lat],
                essential: true,
            });

            // eliminar todos los marcadores existentes
            this.markers.forEach(marker => marker.remove());
            this.markers = [];

            let img = document.createElement('img');
            img.src = this.str_img;

            let markerElement = document.createElement('div')
            markerElement.appendChild(img)

            this.markers.push(new mapboxgl.Marker({ element: markerElement })
                .setLngLat([this.data.device.GPS_lng, this.data.device.GPS_lat])
                .addTo(this.mapModal));

        },

        SaveMap(){

                const data = {
                    name: this.data.device.name,
                    GPS_lat: parseFloat(this.data.device.GPS_lat),
                    GPS_lng: parseFloat(this.data.device.GPS_lng)
                };

                window.emitter.emit('changeLoaderStatus', true, 'orbit')



                axios({
                    method: 'put',
                    url: window.location.origin + '/api/gateway/edit/' + this.user.id + '/' + this.data.device.id,
                    headers: {
                        'Accept': 'application/json',
                        'Content-Type': 'application/json',
                        'Authorization': 'Bearer ' + this.user.token
                    },
                    data: data
                })
                    .then((response) => {

                        window.emitter.emit('changeLoaderStatus', false, 'orbit')

                        this.data.device.GPS_lat = data.GPS_lat
                        this.data.device.GPS_lng = data.GPS_lng

                        this.device_update.GPS_lat = data.GPS_lat
                        this.device_update.GPS_lng = data.GPS_lng

                    })
                    .catch((error) => {
                        window.emitter.emit('changeLoaderStatus', false, 'orbit')

                        if(error.response && error.response.status && error.response.status == 429){
                            window.alert( this.$t('text.Api attempts'));
                        }else{
                            return "Failed!", error;
                        }
                    });

                this.rechargePage();

            /*}else {

                this.data.device.GPS_lat = this.device_update.GPS_lat
                this.data.device.GPS_lng = this.device_update.GPS_lng

            } */

            this.Mapconfig = false;

        },

        CancelMap() {
            this.Mapconfig = false;

            if(this.data.device.GPS_lat != this.device_update.GPS_lat || this.data.device.GPS_lng != this.device_update.GPS_lng){
                this.data.device.GPS_lat = this.device_update.GPS_lat;
                this.data.device.GPS_lng = this.device_update.GPS_lng;
                this.rechargePage();
            }
        },

        check_VMoptions(index) {

            let times = 0;

            this.VMoptions[index].forEach((value, i) => {
                if (typeof value == 'object') {
                    times += 1;
                }
            })

            if (times > 1) {
                return true;
            } else {
                return false;
            }

        },

        selectMultiselectMeter: function (item) {
            this.data.busIdDevice = item

            this.data.names = this.data.meters[item].names
            this.data.units = this.data.meters[item].units

            this.data.data = new Array()
            this.data.dates = new Array()

            if( this.date[0].getFullYear() === this.date[1].getFullYear() &&
                this.date[0].getMonth() === this.date[1].getMonth() &&
                this.date[0].getDate() === this.date[1].getDate()){

                    this.date[0].setHours(0);
                    this.date[0].setMinutes(0);
                    this.date[0].setSeconds(0);

                    this.date[1].setHours(23);
                    this.date[1].setMinutes(59);
                    this.date[1].setSeconds(59);
            }

            let date_start = this.formatDateLocal(this.date[0]);
            let date_end = this.formatDateLocal(this.date[1]);

            axiosApiCalls.methodGet('/gateway/values/period/'+this.user.id+'/'+this.data.device.id+'/'+date_start+'/'+date_end+'/'+item, this.user.token)
            .then((response) => {

                if(response.dates == undefined || response.dates.length == 0){
                    window.emitter.emit(
                        'AppLayout_alert',
                        true,
                        this.$t('general.Gateway without records'),
                        this.$t('text.lacks records period'),
                        '18rem'
                    )

                    this.data.stats = undefined;
                    this.data.daily = undefined;
                    this.data.data = undefined;
                    this.data.dates = undefined;
                    this.data.summary = undefined;
                } else {
                    let stats = {};

                    for (let sensor in response.data) {
                        let serie = Object.values(response.data[sensor]);

                        if (response.decimals !== undefined && response.decimals[sensor] !== undefined && serie.length !== 0) {
                            let max = Math.max(...serie);
                            let min = Math.min(...serie);
                            let average = serie.reduce((acc, val) => acc + val, 0) / serie.length;
                            let max_amplitude = max - min;

                            stats[sensor] = {
                                max: max.toFixed(response.decimals[sensor]),
                                min: min.toFixed(response.decimals[sensor]),
                                avg: average.toFixed(response.decimals[sensor]),
                                max_amplitude: max_amplitude.toFixed(response.decimals[sensor]),
                            };
                        } else if (serie.length !== 0){
                            let max = Math.max(...serie);
                            let min = Math.min(...serie);
                            let average = serie.reduce((acc, val) => acc + val, 0) / serie.length;
                            let max_amplitude = max - min;

                            stats[sensor] = {
                                max: max,
                                min: min,
                                avg: average,
                                max_amplitude: max_amplitude,
                            };
                        }
                    }

                    this.data.stats = stats;

                    this.data.data = response.data;
                    this.data.dates = response.dates;
                    this.data.summary = response.summary;
                    this.data.names = this.data.meters[item].names
                    this.data.units = this.data.meters[item].units
                }

                this.rechargePage();
            })
        },

        selectMultiselectSensor: function (item) {

            this.Charts.forEach((chart, index_c) => {

                if (chart != undefined) {

                    chart.sensors.forEach((sensor, index_s) => {

                        if (sensor == item.id) {

                            this.max[index_c] = parseFloat(this.data.stats[sensor].max)
                            this.min[index_c] = parseFloat(this.data.stats[sensor].min)
                            this.max_amplitude[index_c] = parseFloat(this.data.stats[sensor].max_amplitude)
                            this.avg[index_c] = parseFloat(this.data.stats[sensor].avg)

                        }
                    })
                }
            })
        },

        Save_update(){

            const data = {
                name: document.getElementById('id_name').value,
                GPS_lat: parseFloat(document.getElementById('id_lat').value),
                GPS_lng: parseFloat(document.getElementById('id_lng').value),
            };

            window.emitter.emit('changeLoaderStatus', true, 'orbit')



            axios({
                method: 'put',
                url: window.location.origin + '/api/gateway/edit/' + this.user.id + '/' + this.data.device.id,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
                data: data
            })
                .then((response) => {

                    this.data.device.name = data.name
                    this.data.device.GPS_lat = data.GPS_lat
                    this.data.device.GPS_lng = data.GPS_lng

                    this.device_update = {
                        name: data.name,
                        GPS_lat: data.GPS_lat,
                        GPS_lng: data.GPS_lng
                    }

                    this.rechargePage();

                    window.emitter.emit('changeLoaderStatus', false, 'orbit')


                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                        window.alert( this.$t('text.Api attempts'));
                    }else{
                        return "Failed!", error;
                    }
                });

        },

        Cancel_update(){

            document.getElementById('id_name').value = this.device_update.name;
            document.getElementById('id_lat').value = parseFloat(this.device_update.GPS_lat).toFixed(6);
            document.getElementById('id_lng').value = parseFloat(this.device_update.GPS_lng).toFixed(6);

        },

        getName(chart_info) {

            let name;
            let unit = chart_info.unit;
            let id = chart_info.id

            this.Charts.forEach((chart, index_c) => {

                if(chart != undefined && chart.id == id){

                    if(chart.sensors.length <= 1){
                        for (let idfield in chart.sensors) {

                            if(typeof this.data.names[parseInt(chart.sensors[idfield])] != 'undefined')
                                name = this.data.names[parseInt(chart.sensors[idfield])] + ( chart.unit != '' ? ' (' + chart.unit + ')' : '' );
                        }
                    }else{
                        const nameConcordance = this.nameConcordance(this.data.names, null, chart.sensors, unit)
                        if(nameConcordance != ''){
                            name = nameConcordance;
                        }else{
                            name = this.getNameByUnit(unit);
                        }
                    }
                }
            })

            if(this.data.type_name == 'PARK')
                return '';

            return name;
        },

        getNameByUnit(unit){
            switch (unit) {
                case 'ºC': return this.$t("general.Temperature") + ' (' + unit + ')';
                case '%': return this.$t("general.Ratio") + ' (' + unit + ')';
                case 'dB': return this.$t("general.Decibels") + ' (' + unit + ')';
                case 'V': return this.$t("general.Voltage") + ' (' + unit + ')';
                case 'A': return this.$t("general.Current") + ' (' + unit + ')';
                case 'Hz': return this.$t("general.Frequency") + ' (' + unit + ')';
                case 'h': return this.$t("general.Period") + ' (' + unit + ')';
                case 'ug': return this.$t("general.Microgram") + ' (' + unit + ')';
                default: return '';
            }
        },

        methodFormatDatetoDatepicker() {
            return generateDateDatepicker(this.date)
        },

        generateDateDatepicker(date) {

            var day1 = date[0].getDate();
            var month1 = date[0].getMonth() + 1;
            var year1 = date[0].getFullYear();
            var hour1 = date[0].getHours();
            var min1 = date[0].getMinutes();

            if(month1.toString().length == 1) month1 = '0' + month1;
            if(day1.toString().length == 1) day1 = '0' + day1;
            if(hour1.toString().length == 1) hour1 = '0' + hour1;
            if(min1.toString().length == 1) min1 = '0' + min1;

            var day2 = date[1].getDate();
            var month2 = date[1].getMonth() + 1;
            var year2 = date[1].getFullYear();
            var hour2 = date[1].getHours();
            var min2 = date[1].getMinutes();

            if(month2.toString().length == 1) month2 = '0' + month2;
            if(day2.toString().length == 1) day2 = '0' + day2;
            if(hour2.toString().length == 1) hour2 = '0' + hour2;
            if(min2.toString().length == 1) min2 = '0' + min2;

            return `${year1}/${month1}/${day1} ${hour1}:${min1} `+ this.$t("general.To") +` ${year2}/${month2}/${day2} ${hour2}:${min2}`;
        },

        methodUpdateDate(modelData) {

            if( modelData[0].getFullYear() === modelData[1].getFullYear() &&
                modelData[0].getMonth() === modelData[1].getMonth() &&
                modelData[0].getDate() === modelData[1].getDate()){

                    modelData[0].setHours(0);
                    modelData[0].setMinutes(0);
                    modelData[0].setSeconds(0);

                    modelData[1].setHours(23);
                    modelData[1].setMinutes(59);
                    modelData[1].setSeconds(59);
            }

            let date_start = this.formatDateLocal(modelData[0]);
            let date_end = this.formatDateLocal(modelData[1]);

            axiosApiCalls.methodGet('/gateway/values/period/'+this.user.id+'/'+this.data.device.id+'/'+date_start+'/'+date_end, this.user.token)
            .then((response) => {

                if(response.dates == undefined || response.dates.length == 0){
                    window.emitter.emit(
                        'AppLayout_alert',
                        true,
                        this.$t('general.Gateway without records'),
                        this.$t('text.lacks records period'),
                        '18rem'
                    )

                    this.data.stats = undefined;
                    this.data.daily = undefined;
                    this.data.data = undefined;
                    this.data.dates = undefined;
                    this.data.summary = undefined;
                } else {

                    let stats = {};

                    if(response.type_name != 'PARK'){
                        for (let sensor in response.data) {
                            let serie = Object.values(response.data[sensor]);

                            if (response.decimals !== undefined && response.decimals[sensor] !== undefined && serie.length !== 0) {
                                let max = Math.max(...serie);
                                let min = Math.min(...serie);
                                let average = serie.reduce((acc, val) => acc + val, 0) / serie.length;
                                let max_amplitude = max - min;

                                stats[sensor] = {
                                    max: max.toFixed(response.decimals[sensor]),
                                    min: min.toFixed(response.decimals[sensor]),
                                    avg: average.toFixed(response.decimals[sensor]),
                                    max_amplitude: max_amplitude.toFixed(response.decimals[sensor]),
                                };
                            } else if (serie.length !== 0){
                                let max = Math.max(...serie);
                                let min = Math.min(...serie);
                                let average = serie.reduce((acc, val) => acc + val, 0) / serie.length;
                                let max_amplitude = max - min;

                                stats[sensor] = {
                                    max: max,
                                    min: min,
                                    avg: average,
                                    max_amplitude: max_amplitude,
                                };
                            }
                        }
                    } else {
                        let max = null;
                        let min = null;
                        let avg = null;
                        let count = 0;

                        response.data.forEach((register, key) => {

                            if (register['lecture_value'] == 1) {
                                const timestampStart = new Date(response.dates[key]).getTime();

                                let timestampEnd;
                                if (response.dates[key - 1]) {
                                    timestampEnd = new Date(response.dates[key - 1]).getTime();
                                } else {
                                    timestampEnd = new Date().getTime();
                                }

                                const duration = timestampEnd - timestampStart;

                                if (duration < min || min === null) {
                                    min = duration;
                                }

                                if (duration > max || max === null) {
                                    max = duration;
                                }

                                avg = (avg || 0) + duration;
                                count += 1;
                            }
                        })

                        if (max === null) {
                            max = 0;
                        }

                        if (min === null) {
                            min = 0;
                        }

                        if (avg === null) {
                            avg = 0;
                        }

                        stats = {
                            'max': max,
                            'min': min,
                            'avg': count !== 0 ? avg / count : avg,
                        };
                    }

                    this.data.stats = stats;

                    this.data.daily = response.daily;
                    this.data.data = response.data;
                    this.data.dates = response.dates;
                    this.data.summary = response.summary;

                }

                this.rechargePage();

                return response;
            })
        },

        changeChartsMode(input){
            if (this.charts_mode != input){
                window.emitter.emit('changeLoaderStatus', true, 'orbit')
                this.charts_mode = input;
                this.rechargePage()
            }
        },

        nameConcordance(names, sensor, chartSensors, unit){

            if (sensor != null){
                const baseName = names[sensor].split(' ')[0].toLowerCase();

                for (let sensor_c of chartSensors) {
                    const currentName = names[sensor_c].split(' ')[0].toLowerCase();  // Obtener la primera palabra de names[sensor_c]

                    if (baseName !== currentName) {
                        return false;  // Si las primeras palabras no coinciden, retornar false
                    }
                }

                return true;  // Si todas las primeras palabras coinciden, retornar true

            } else {

                var baseName = '';

                for (let sensor_c of chartSensors) {

                    const currentName = names[sensor_c];  // Obtener la primera palabra de names[sensor_c]

                    if (baseName == '') {
                        baseName = currentName;
                    } else {
                        let matchingPart = '';

                        for (let i = 0; i < Math.min(baseName.length, currentName.length); i++) {
                            if (baseName[i] === currentName[i]) {
                                matchingPart += baseName[i];  // Agregar el carácter coincidente al string matchingPart
                            } else {
                                break;  // Si encontramos un carácter que no coincide, salimos del bucle
                            }
                        }

                        baseName = matchingPart;  // Actualizar baseName con la parte coincidente
                    }
                }

                var wordsArray = baseName.split(' ')

                while(typeof wordsArray[wordsArray.length - 1] != 'undefined' && wordsArray[wordsArray.length - 1].length <= 3){
                    if(wordsArray.length > 0){
                        wordsArray.pop()
                    }
                }

                if(wordsArray.length == 0) {
                    return this.getNameByUnit(unit);
                } else {
                    return wordsArray.join(' ') + ((unit != '') ? ' (' + unit + ')' : '')
                }

            }
        },
    },
}
</script>
